# about.py

__version__ = "0.1.0"
__package__ = "speech_reading_trainer"
__program_name__ = "speech-reading-trainer"
__author__ = "Fernando Pujaico Rivera"
__email__  = "fernando.pujaico.rivera@gmail.com"
__description__ = "A desktop application that helps users improve reading fluency and pronunciation by combining text-to-speech playback, voice recording, automatic transcription, and real-time accuracy feedback."
__url_source__  = "https://github.com/trucomanx/SpeechReadingTrainer"
__url_doc__  = "https://github.com/trucomanx/SpeechReadingTrainer/tree/main/doc"
__url_funding__ = "https://trucomanx.github.io/en/funding.html"
__url_bugs__    = "https://github.com/trucomanx/SpeechReadingTrainer/issues"
